#define _GNU_SOURCE
#include <alsa/asoundlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>
#include "ogg.h"

OggVorbis_File vf;
int current_bitstream = 0;
FILE* fp;
struct stat file_stat;

int copy_to_ram = 1;
char* audio;

int ogg_load_file(const char* filename){
	if ( (fp = fopen(filename, "r")) != NULL)
	{
		if (copy_to_ram){
			if (stat(filename, &file_stat) != 0){
				fprintf(stderr, "Cannot determine file size!\n"); 
				return -1; // error while getting file attributes
			}
			audio = malloc(file_stat.st_size);
			fread(audio, file_stat.st_size, 1, fp);
			if (ferror(fp) != 0) {
				fclose(fp);
				fprintf(stderr, "Cannot read from file %s!\n", filename); 
				return -2; // failed to read from file
			}
			fclose(fp);
			fp = fmemopen(audio, file_stat.st_size, "r");
		}
		if(ov_open(fp, &vf, NULL, 0) < 0) {
			fprintf(stderr,"Input does not appear to be an Ogg bitstream.\n");
			fclose(fp);
			return(-1);
		}
		// success!
		return 0;				
	}
	fprintf(stderr, "Cannot open file %s!\n", filename); 
	return -3; // failed to open file
}

int ogg_close_file(){
	return ov_clear(&vf);
}

int ogg_get_resolution(){
	return 16;
}

int ogg_get_sampling_rate(){
	vorbis_info *vi=ov_info(&vf,-1);
	return vi->rate;

}

int ogg_get_channels(){
	vorbis_info *vi=ov_info(&vf,-1);
	return vi->channels;
}

snd_pcm_format_t ogg_get_sound_format(){
	return SND_PCM_FORMAT_S16_LE;
}

int ogg_fill_buffer(char* buf, unsigned int bytes){ 
	int bytes_read = 0;
	int ret = 0;
	while (bytes > 0){
		ret = ov_read(&vf, buf, bytes, OGG_LITTLE_ENDIAN, OGG_16BIT, OGG_SIGNED, &current_bitstream);
		if (ret <= 0) {
			exit(-2);
		}
		bytes_read += ret;
		bytes -= ret;
		buf += ret;
	}
	return bytes_read;
}
